@@ -39,7 +39,7 @@ use warnings;
 use utf8;
 use 5.008008; # minumum perl version is 5.8.8
 
-our $VERSION = '1.103';
+our $VERSION = '1.110';
 our $VERSION_MATURITY = "";
 
 use constant ONE_MB => 1024*1024;
@@ -94,11 +94,16 @@ sub print_system_modules_version
 
 sub load_all_dynamic_modules
 {
-	# we load here all dynamically loaded modules, to check that installation is correct.
 	require App::MtAws::Command::Sync;
 	require App::MtAws::Command::Retrieve;
 	require App::MtAws::Command::CheckLocalHash;
 	require App::MtAws::Command::DownloadInventory;
+}
+
+sub check_all_dynamic_modules
+{
+	# we load here all dynamically loaded modules, to check that installation is correct.
+	load_all_dynamic_modules();
 	check_module_versions;
 }
 
@@ -361,7 +366,7 @@ END
 ## use Test::Tabs
 
 	} elsif ($action eq 'version') {
-		load_all_dynamic_modules();
+		check_all_dynamic_modules();
 		print "mt-aws-glacier version: $VERSION $VERSION_MATURITY\n";
 		print "Perl Version: $]\n";
 		print_system_modules_version();
