@@ -20,7 +20,7 @@
 
 package App::MtAws::ConfigDefinition;
 
-our $VERSION = "1.103";
+our $VERSION = "1.110";
 
 use strict;
 use warnings;
@@ -101,14 +101,35 @@ sub check_dir_or_relname
 				custom('relfilename', do {
 					validate 'dir', 'filename';
 					if (valid('dir') && valid('filename')) {
-						my $relfilename = File::Spec->abs2rel(value('filename'), value('dir'));
-						if ($relfilename =~ m!^\.\./!) {
-							error(message('filename_inside_dir',
-								'File specified with "option a" should be inside directory specified in %option b%'),
-								a => 'filename', b => 'dir'),
-							undef;
+
+						my $b_dir = binary_abs_path(binaryfilename value('dir'));
+						my $b_file = binary_abs_path(binaryfilename value('filename'));
+
+						if (!defined $b_dir) {
+							error(message('cannot_resolve_dir',
+								'Directory specified with "%option a%" cannot be resolved to full path'),
+								a => 'dir'), undef
+						} elsif (!defined $b_file) {
+							error(message('cannot_resolve_file',
+								'File specified with "%option a%" cannot be resolved to full path'),
+								a => 'filename'), undef;
 						} else {
-							$relfilename
+							my $relfilename = characterfilename abs2rel($b_file, $b_dir, allow_rel_base => 0, use_filename_encoding => 0);
+
+							my $dir = value('dir');
+							$dir =~ s!/$!!; # just in case
+
+							confess "something wrong with relative-absolute paths"
+								unless file_inodev(value('filename')) eq file_inodev($dir."/".$relfilename);
+
+							if (!is_relative_filename($relfilename)) {
+								error(message('filename_inside_dir',
+									'File specified with "option a" should be inside directory specified in %option b%'),
+									a => 'filename', b => 'dir'),
+								undef;
+							} else {
+								$relfilename
+							}
 						}
 					} else {
 						undef;
@@ -335,6 +356,8 @@ sub get_config
 			validation $_, $must_be_an_integer, stop => 1, sub { $_ =~ /^\d+$/ };
 			validation $_, message('Part size must be power of two'), sub { ($_ != 0) && (($_ & ($_ - 1)) == 0) };
 			validation $_, message('%option a% must be less or equal to 4096'), sub { $_ <= 4096 };
+			validation $_, message('On 32 bit systems, when Digest::SHA module version < 5.62, %option a% must be less or equal to 256'),
+				sub { !is_digest_sha_broken_for_large_data() || $_ <= 256 };
 		}
 		for (option('segment-size', type => 'i')) {
 			validation $_, $must_be_an_integer, stop => 1, sub { $_ =~ /^\d+$/ };
