@@ -20,15 +20,15 @@
 
 package App::MtAws::Journal;
 
-our $VERSION = '1.103';
+our $VERSION = '1.110';
 
 use strict;
 use warnings;
 use utf8;
 
 
-use File::Find ;
-use File::Spec;
+use File::Find;
+use File::Spec 3.12;
 use Encode;
 use Carp;
 use IO::Handle;
@@ -51,6 +51,12 @@ sub new
 
 	$self->{journal_encoding} ||= 'UTF-8';
 
+	if (defined $self->{root_dir}) {
+		# copied from File::Spec::catfile
+		$self->{canon_root_dir} = File::Spec->catdir($self->{root_dir});
+		$self->{canon_root_dir} .= "/" unless substr($self->{canon_root_dir},-1) eq "/";
+	}
+
 	defined($self->{journal_file}) || confess;
 	$self->{journal_h} = {};
 	$self->{archive_h} = {};
@@ -292,6 +298,7 @@ sub _write_line
 # Reading file listing
 #
 
+
 sub read_files
 {
 	my ($self, $mode, $max_number_of_files) = @_;
@@ -309,6 +316,7 @@ sub read_files
 	# TODO: find better workaround than "-s"
 	$File::Find::prune = 0;
 	$File::Find::dont_use_nlink = !$self->{leaf_optimization};
+
 	File::Find::find({ wanted => sub {
 		if ($self->_listing_exceeed_max_number_of_files($max_number_of_files)) {
 			$File::Find::prune = 1;
@@ -325,7 +333,8 @@ sub read_files
 
 		if (-d) {
 			my $dir = character_filename($_);
-			my $reldir = File::Spec->abs2rel($dir, $self->{root_dir});
+			$dir =~ s!/$!!; # make sure there is no trailing slash. just in case.
+			my $reldir = abs2rel($dir, $self->{root_dir}, allow_rel_base => 1);
 			if ($self->{filter} && $reldir ne '.') {
 				my ($match, $matchsubdirs) = $self->{filter}->check_dir($reldir."/");
 				if (!$match && $matchsubdirs) {
@@ -335,7 +344,7 @@ sub read_files
 		} else {
 			# file can be not existing here (i.e. dangling symlink)
 			my $filename = character_filename(my $binaryfilename = $_);
-			my $orig_relfilename = File::Spec->abs2rel($filename, $self->{root_dir});
+			my $orig_relfilename = abs2rel($filename, $self->{root_dir}, allow_rel_base => 1);
 			if (!$self->{filter} || $self->{filter}->check_filenames($orig_relfilename)) {
 				if ($self->_is_file_exists($binaryfilename)) {
 					my $relfilename;
@@ -392,8 +401,12 @@ sub _is_file_exists
 sub absfilename
 {
 	my ($self, $relfilename) = @_;
-	confess unless defined($self->{root_dir});
-	return File::Spec->rel2abs($relfilename, $self->{root_dir});
+	confess unless defined($self->{canon_root_dir});
+
+	# Originally it was: File::Spec->rel2abs($relfilename, $self->{root_dir});
+
+	# TODO: maybe add File::Spec->canonpath() and fix absfilename_correct=./dirA/file3 ?
+	$self->{canon_root_dir}.$relfilename;
 }
 
 
